/***************************************************************************
 *cr                                                                       
 *cr            (C) Copyright 1995 The Board of Trustees of the           
 *cr                        University of Illinois                       
 *cr                         All Rights Reserved                        
 *cr                                                                   
 ***************************************************************************/

/***************************************************************************
 * RCS INFORMATION:
 *
 *	$RCSfile: Light.h,v $
 *	$Author: billh $	$Locker:  $		$State: Exp $
 *	$Revision: 1.8 $	$Date: 1995/05/11 23:02:01 $
 *
 ***************************************************************************
 * DESCRIPTION:
 *
 * A Displayable3D object which represents a light source at infinity, along
 * some line from the origin, of some color.
 *
 ***************************************************************************/
#ifndef LIGHT_H
#define LIGHT_H

#include "Displayable.h"
#include "DisplayDevice.h"
#include "DispCmds.h"
#include "Matrix4.h"


class Light : public Displayable3D {

public:
  // color of the light defining the axes
  float color[3];
  
  // position of the light (rotated from this position by value of trans mat)
  float pos[3];

private:
  // is the light on or off?
  int lightOn;

  // is the light highlight on or off?
  int highlightOn;

  // do we need to define the light? or to turn it on/off?
  int need_define, need_onoff, need_highlight;

  // do we need to clean up after a draw where colors were defined?
  int needDefineCleanup;

  // which light we are
  int num;

  // regenerate the command list
  void create_cmdlist();

  // identity matrix for this to use
  Matrix4 ident;

  // load matrix command
  DispCmdLoad *loadIdentMat;

  // useful display commands
  DispCmdLightDef deflight;
  DispCmdLightOnOff onofflight;
  DispCmdColorRGB hcol;
  DispCmdLine hline;
  DispCmdTextPos htxtpos;
  DispCmdText htxt;
  DispCmdLineType cmdLineType;
  DispCmdLineWidth cmdLineWidth;

public:
  // constructor: the Scene to register this with, the light number,
  // the color, and the position
  Light(Scene *, int, float *, float *);
  virtual ~Light(void);

  // indicate we must change the current settings
  void redefine(void) { need_define = TRUE; }

  // set if we are highlighting
  int light_highlighted(void) const { return highlightOn; }
  void highlight(int h) { highlightOn = h; need_highlight = TRUE; }

  // override the on/off commands since the light will always need to be
  // taken care of
  int light_displayed(void) const { return lightOn; }
  virtual void off(void);
  virtual void on(void);

  // special rotation commands for this object the rotating commands
  void add_rotation(float x, char axis);
  void add_rotation(Matrix4 &);
  void set_rotation(Matrix4 &);

  //
  // public virtual routines
  //
  
  // prepare for drawing ... do any updates needed right before draw.
  virtual void prepare(DisplayDevice *);

};

#endif

