/***************************************************************************
 *cr                                                                       
 *cr            (C) Copyright 1995 The Board of Trustees of the           
 *cr                        University of Illinois                       
 *cr                         All Rights Reserved                        
 *cr                                                                   
 ***************************************************************************/

/***************************************************************************
 * RCS INFORMATION:
 *
 *	$RCSfile: MoleculeRemote.h,v $
 *	$Author: billh $	$Locker:  $		$State: Exp $
 *	$Revision: 1.8 $	$Date: 1995/11/02 02:04:41 $
 *
 ***************************************************************************
 * DESCRIPTION:
 *
 * A MoleculeRemote is inherited from Molecule and contains a Remote;
 * it is a Molecule which is representing a remote simulation, from which
 * it gets parameters and coordinates.
 *
 ***************************************************************************/
#ifndef MOLECULEREMOTE_H
#define MOLECULEREMOTE_H

#include "Molecule.h"
#include "Remote.h"
class DrawPatch;
class DrawForce;


class MoleculeRemote : public Molecule {

public:
  // our remote connection
  Remote *rem;

private:
  // name of path and filename containing structure of molecule
  char *strName, *strPath;

  // number of the timestep we're currently at
  int Step;

  // number of frames we've received from remote connection
  int framesReceived;

  // does this molecule have any patch data?
  int hasPatchDisplay;

  // patch and force display representations
  DrawPatch *drawPatch;
  DrawForce *drawForce;

  // do main constructor tasks
  void do_construct(Remote *);

  // create structure from remote connection data
  int createFromRemote(Remote *);

public:
  // class constructor/destructor
  MoleculeRemote(Remote *, Scene *);
  MoleculeRemote(Remote *, Displayable *);
  virtual ~MoleculeRemote(void);

  // current step of simulation
  int step(void) { return Step; }

  // return the path and name of this molecule
  virtual char *file_path(void);
  virtual char *file_name(void);

  //
  // routines for displaying patch data (if available)
  // static data about names for display methods found at end of this file
  //

  // return whether this molecule has any patch data to display
  int has_patch_display(void) { return hasPatchDisplay; }

  // get/change the method for displaying patches
  int patch_display_method(void);
  int patch_display_method(int);

  //
  // public virtual routines for Molecule
  //
  
  // creates the molecule, using the info provided in the constructor.
  // returns success, and also calls parent class create routines
  virtual int create(void);


  //
  // public virtual routines for Displayable
  //
  
  // prepare for drawing ... do any updates needed right before draw.
  //  ... this updates the animation
  virtual void prepare(DisplayDevice *);

  // these are called from prepare.  They check and send new information to
  // the Remote simulation
private:
  void send_new_forces(void);
  void send_new_coords(void);

};

#endif

